/****************************************************************************

    FILE:    hextool.cpp

 ****************************************************************************

    LICENSE:

    HexTool -  a simple program to convert numbers on the fly
    Copyright (C) 2003 Ryan Harkin and 2014 Christian Weniger; www.harkin.org/HexTool

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/
#include <stdio.h>

#include <common/debug.h>
#include <common/handlebinarydigit.h>
#include <common/numbits.h>
#include <common/register.h>
#include <common/registeroperation.h>
#include <common/signed.h>
#include <common/value.h>
#include <optionsdialog.h>
#include "hextool.h"
#if defined(Q_WS_MAEMO_5)
    #include "ui_hextool.h"
    #include <QMaemo5InformationBox>
#else
    #include "ui_hextool_desk.h"
#endif

#include <QtCore/QCoreApplication>
//#include <qmath.h>
#include <QMessageBox>
#include <QSettings>
#include <QDir>
#include <QDebug>
#include <QCloseEvent>
//#include <QPixmap>

HexTool::HexTool(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::HexTool)
{
    optsl=false;
    optsignext=false;
    SetSigned(UNSIGNED);
    SetNumBits(NUMBITS_32);
    SetRegister(REG_A);

    ui->setupUi(this);

    /************setting IDs for buttons*************/
    ui->buttonGroup->setId(ui->pushButton_bit1, 1);
    ui->buttonGroup->setId(ui->pushButton_bit2, 2);
    ui->buttonGroup->setId(ui->pushButton_bit3, 3);
    ui->buttonGroup->setId(ui->pushButton_bit4, 4);
    ui->buttonGroup->setId(ui->pushButton_bit5, 5);
    ui->buttonGroup->setId(ui->pushButton_bit6, 6);
    ui->buttonGroup->setId(ui->pushButton_bit7, 7);
    ui->buttonGroup->setId(ui->pushButton_bit8, 8);
    ui->buttonGroup->setId(ui->pushButton_bit9, 9);
    ui->buttonGroup->setId(ui->pushButton_bit10, 10);
    ui->buttonGroup->setId(ui->pushButton_bit11, 11);
    ui->buttonGroup->setId(ui->pushButton_bit12, 12);
    ui->buttonGroup->setId(ui->pushButton_bit13, 13);
    ui->buttonGroup->setId(ui->pushButton_bit14, 14);
    ui->buttonGroup->setId(ui->pushButton_bit15, 15);
    ui->buttonGroup->setId(ui->pushButton_bit16, 16);
    ui->buttonGroup->setId(ui->pushButton_bit17, 17);
    ui->buttonGroup->setId(ui->pushButton_bit18, 18);
    ui->buttonGroup->setId(ui->pushButton_bit19, 19);
    ui->buttonGroup->setId(ui->pushButton_bit20, 20);
    ui->buttonGroup->setId(ui->pushButton_bit21, 21);
    ui->buttonGroup->setId(ui->pushButton_bit22, 22);
    ui->buttonGroup->setId(ui->pushButton_bit23, 23);
    ui->buttonGroup->setId(ui->pushButton_bit24, 24);
    ui->buttonGroup->setId(ui->pushButton_bit25, 25);
    ui->buttonGroup->setId(ui->pushButton_bit26, 26);
    ui->buttonGroup->setId(ui->pushButton_bit27, 27);
    ui->buttonGroup->setId(ui->pushButton_bit28, 28);
    ui->buttonGroup->setId(ui->pushButton_bit29, 29);
    ui->buttonGroup->setId(ui->pushButton_bit30, 30);
    ui->buttonGroup->setId(ui->pushButton_bit31, 31);
    ui->buttonGroup->setId(ui->pushButton_bit32, 32);
    ui->rbuttonGroup_bitsize->setId(ui->radioButton_8bit, 8);
    ui->rbuttonGroup_bitsize->setId(ui->radioButton_16bit, 16);
    ui->rbuttonGroup_bitsize->setId(ui->radioButton_32bit, 32);
    ui->rbuttonGroup_reg->setId(ui->radioButton_registerA, 1);
    ui->rbuttonGroup_reg->setId(ui->radioButton_registerB, 2);
    ui->rbuttonGroup_reg->setId(ui->radioButton_registerC, 3);
    connect(ui->buttonGroup, SIGNAL(buttonClicked(int)), this, SLOT(groupButton_clicked(int)));
    UpdateDisplay();
    /********setting the menu********************************/
    QAction *clear = new QAction("ClearAll",this);
    QAction *about = new QAction("About",this);
    QAction *save = new QAction("Save",this);
    QAction *load = new QAction("Load",this);
    QAction *help = new QAction("Help",this);
    QAction *opt = new QAction("Options",this);
    this->menuBar()->addAction(clear);
    this->menuBar()->addAction(load);
    this->menuBar()->addAction(save);
    this->menuBar()->addAction(opt);
    this->menuBar()->addAction(help);
    this->menuBar()->addAction(about);
    connect(clear, SIGNAL(triggered()), this, SLOT(clearAll()));
    connect(about, SIGNAL(triggered()), this, SLOT(about()));
    connect(save, SIGNAL(triggered()), this, SLOT(save_ini()));
    connect(load, SIGNAL(triggered()), this, SLOT(load_ini()));
    connect(help, SIGNAL(triggered()), this, SLOT(help()));
    connect(opt, SIGNAL(triggered()), this, SLOT(optDialog()));
    /********automatic load settings**************************/
    QFileInfo fi(QDir::homePath() + "/.hextool/settings.ini");
    if (fi.exists())
    {
        QSettings settings(QDir::homePath() + "/.hextool/settings.ini", QSettings::IniFormat);
        settings.beginGroup("Options");
        optsl=settings.value("optsl").toBool();
        settings.endGroup();
        if (optsl)
        {
            load_ini();
        }
    }
    /**********************************************************/
}

HexTool::~HexTool()
{
    delete ui;
}

void HexTool::setOrientation(ScreenOrientation orientation)
{
#if defined(Q_OS_SYMBIAN)
    // If the version of Qt on the device is < 4.7.2, that attribute won't work
    if (orientation != ScreenOrientationAuto) {
        const QStringList v = QString::fromAscii(qVersion()).split(QLatin1Char('.'));
        if (v.count() == 3 && (v.at(0).toInt() << 16 | v.at(1).toInt() << 8 | v.at(2).toInt()) < 0x040702) {
            qWarning("Screen orientation locking only supported with Qt 4.7.2 and above");
            return;
        }
    }
#endif // Q_OS_SYMBIAN

    Qt::WidgetAttribute attribute;
    switch (orientation) {
#if QT_VERSION < 0x040702
    // Qt < 4.7.2 does not yet have the Qt::WA_*Orientation attributes
    case ScreenOrientationLockPortrait:
        attribute = static_cast<Qt::WidgetAttribute>(128);
        break;
    case ScreenOrientationLockLandscape:
        attribute = static_cast<Qt::WidgetAttribute>(129);
        break;
    default:
    case ScreenOrientationAuto:
        attribute = static_cast<Qt::WidgetAttribute>(130);
        break;
#else // QT_VERSION < 0x040702
    case ScreenOrientationLockPortrait:
        attribute = Qt::WA_LockPortraitOrientation;
        break;
    case ScreenOrientationLockLandscape:
        attribute = Qt::WA_LockLandscapeOrientation;
        break;
    default:
    case ScreenOrientationAuto:
        attribute = Qt::WA_AutoOrientation;
        break;
#endif // QT_VERSION < 0x040702
    };
    setAttribute(attribute, true);
}

void HexTool::showExpanded()
{
#if defined(Q_OS_SYMBIAN) || defined(Q_WS_SIMULATOR)
    showFullScreen();
#elif defined(Q_WS_MAEMO_5)
    showMaximized();
#else
    show();
#endif
}

/****************************************************************************

    Qt Event Handler

    control:    hextool widget
    signal:     closed, exit

    It is used for automatic save during exit.

****************************************************************************/
void HexTool::closeEvent(QCloseEvent *event)
{
    if (optsl)
    {
        save_ini();
    }
    else
    {
        QFileInfo fi(QDir::homePath() + "/.hextool/settings.ini");
        if (fi.exists())
        {
            QSettings settings(QDir::homePath() + "/.hextool/settings.ini", QSettings::IniFormat);
            settings.beginGroup("Options");
            settings.setValue("optsl", optsl);
            settings.endGroup();
        }

    }

    event->accept();
}
/****************************************************************************

    Update Display function

    Refreshes the gui.
//QString::number(value,8) //octal
****************************************************************************/
void HexTool::UpdateDisplay()
{
    static bool updatingDisplay = false;

    if (!updatingDisplay)
    {
    updatingDisplay = true;

    int value = GetValue(REG_CURRENT);
#ifdef QT_DEBUG
    qDebug() << "value from current reg:" << QString::number(value);
#endif

    char str[128];
    if (GetSigned()==UNSIGNED)
    {
        sprintf(str, "%u", value);

#ifdef QT_DEBUG
        qDebug() << "dec char128 unsigned:" << str;
#endif
    }
    else
    {
         switch (GetNumBits())
         {
             case NUMBITS_8:     sprintf(str, "%d", (qint8) value);    break;
             case NUMBITS_16:    sprintf(str, "%d", (qint16)value);    break;
             case NUMBITS_32:    sprintf(str, "%d", (qint32)  value);    break;
         }
#ifdef QT_DEBUG
    qDebug() << "dec char128 signed:" << str;
#endif
    }
    ui->lineEdit_dec->setText(str);
    sprintf(str, "%X", value);
    ui->lineEdit_hex->setText(str);
    sprintf(str, "%o", value);
    ui->lineEdit_oct->setText(str);

    sprintf(str, "%d", (unsigned char)((value & 0x000000FF) >> 0));
    ui->lineEdit_ipaddress0->setText(str);
    sprintf(str, "%d", (unsigned char)((value & 0x0000FF00) >> 8));
    ui->lineEdit_ipaddress1->setText(str);
    sprintf(str, "%d", (unsigned char)((value & 0x00FF0000) >> 16));
    ui->lineEdit_ipaddress2->setText(str);
    sprintf(str, "%d", (unsigned char)((value & 0xFF000000) >> 24));
    ui->lineEdit_ipaddress3->setText(str);

    sprintf(str, "%c", (unsigned char)((value & 0x000000FF) >> 0));
    ui->lineEdit_ascii0->setText(str);
    sprintf(str, "%c", (unsigned char)((value & 0x0000FF00) >> 8));
    ui->lineEdit_ascii1->setText(str);
    sprintf(str, "%c", (unsigned char)((value & 0x00FF0000) >> 16));
    ui->lineEdit_ascii2->setText(str);
    sprintf(str, "%c", (unsigned char)((value & 0xFF000000) >> 24));
    ui->lineEdit_ascii3->setText(str);

    ////////////////////////////////////////////////////////////////////////
    int bitVal;

    for (int bit=0; bit<8; bit++)
    {
        bitVal = (value & (1<<bit))?1:0;
        sprintf(str, "%d", bitVal);
        ui->buttonGroup->button(bit+1)->setText(str);
    }

    if (GetNumBits() != 8)
    {

        ui->frame_bit16->setDisabled(false);
        ui->lineEdit_ascii1 ->setDisabled(false);
        ui->lineEdit_ipaddress1 ->setDisabled(false);

        for (int bit=8; bit<16; bit++)
        {
            bitVal = (value & (1<<bit))?1:0;
            sprintf(str, "%d", bitVal);
            ui->buttonGroup->button(bit+1)->setText(str);
        }

        if (GetNumBits() != 16)
        {

            ui->frame_bit32->setDisabled(false);
            ui->lineEdit_ascii3->setDisabled(false);
            ui->lineEdit_ascii2->setDisabled(false);
            ui->lineEdit_ipaddress3->setDisabled(false);
            ui->lineEdit_ipaddress2->setDisabled(false);

            for (int bit=16; bit<32; bit++)
            {
                bitVal = (value & (1<<bit))?1:0;
                sprintf(str, "%d", bitVal);
                ui->buttonGroup->button(bit+1)->setText(str);
            }
        }
        else
        {

            for (int i=1; i<=16; i++)
            {
                ui->buttonGroup->button(i+16)->setText("");
            }

            ui->frame_bit32->setDisabled(true);
            ui->lineEdit_ascii3->setDisabled(true);
            ui->lineEdit_ascii2->setDisabled(true);
            ui->lineEdit_ipaddress3->setDisabled(true);
            ui->lineEdit_ipaddress2->setDisabled(true);
        }
    }
    else
    {

        for (int i=1; i<=24; i++)
        {
            ui->buttonGroup->button(i+8)->setText("");
        }

        ui->frame_bit16->setDisabled(true);
        ui->frame_bit32->setDisabled(true);
        ui->lineEdit_ascii1->setDisabled(true);
        ui->lineEdit_ipaddress1->setDisabled(true);
        ui->lineEdit_ascii3->setDisabled(true);
        ui->lineEdit_ascii2->setDisabled(true);
        ui->lineEdit_ipaddress3->setDisabled(true);
        ui->lineEdit_ipaddress2->setDisabled(true);
    }
    updatingDisplay = false;
    }
}
/****************************************************************************

    Qt Signal Handler and toggle bit function

    control: pushbuttongroup
    signal: clicked

    Toggles single bits

****************************************************************************/

void HexTool::groupButton_clicked(int bit) { ToggleBit(bit-1); }

void HexTool::ToggleBit(int bit)
{
    HandleBinaryDigit(bit);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    lineEdit_dec
    signal:     textChanged

    The decimal number edit box has changed

****************************************************************************/
void HexTool::on_lineEdit_dec_textChanged()
{
    int value;
    if (GetSigned()==UNSIGNED)
    {
        value = (int)ui->lineEdit_dec->text().toUInt();
    }
    else
    {
        value = (int)ui->lineEdit_dec->text().toInt();
    }
    SetValue(value, REG_CURRENT);
    UpdateDisplay();
}
/****************************************************************************

    Qt Signal Handler

    control:    lineEdit_oct
    signal:     textChanged

    The octal number edit box has changed

****************************************************************************/
void HexTool::on_lineEdit_oct_textChanged()
{
    int value;
    value = (int)ui->lineEdit_oct->text().toUInt(0,8);
    SetValue(value, REG_CURRENT);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    lineEdit_hex
    signal:     textChanged

    The hexadecimal number edit box has changed

****************************************************************************/
void HexTool::on_lineEdit_hex_textChanged()
{
    int value;
    value = (int)ui->lineEdit_hex->text().toUInt(0,16);
    SetValue(value, REG_CURRENT);
    UpdateDisplay();
}
/****************************************************************************

    Qt Signal Handler

    control:    pushButton_clr
    signal:     clicked

    The clr button was clicked clears current register

****************************************************************************/
void HexTool::on_pushButton_clr_clicked()
{
    int value;
    value=0;
    SetValue(value, REG_CURRENT);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    radioButton_8bit
    signal:     pressed

    The radio button for 8 bit mode has been selected (aka pressed)

****************************************************************************/
void HexTool::on_radioButton_8bit_clicked()
{
    SetNumBits(NUMBITS_8);
    UpdateDisplay();
    //ui->lineEdit_hex->setMaxLength(2);
    /*if (ui->checkBox_signed->checkState()) ui->lineEdit_dec->setMaxLength(4);
    else ui->lineEdit_dec->setMaxLength(3);*/
    QString svalue;
    svalue=ui->lineEdit_hex->text();
    int value;
    value = (int)ui->lineEdit_hex->text().toUInt(0,16);
    SetValue(value, REG_CURRENT);
}

/****************************************************************************

    Qt Signal Handler

    control:    radioButton_16bit
    signal:     pressed

    The radio button for 16 bit mode has been selected (aka pressed)

****************************************************************************/
void HexTool::on_radioButton_16bit_clicked()
{
    int bitsize;
    QString svalue;
    int value;

    bitsize=GetNumBits();
    if (bitsize==8 && ui->checkBox_signed->checkState() && optsignext)
    {
        SetNumBits(NUMBITS_16);
        svalue="FF"+ui->lineEdit_hex->text();
        //ui->lineEdit_hex->setText(svalue);
        value=(int)svalue.toUInt(0,16);
        SetValue(value, REG_CURRENT);
#ifdef QT_DEBUG
        qDebug() << "changed value:" << QString::number(value);
#endif
    }
    else
    {
        SetNumBits(NUMBITS_16);
    }


    UpdateDisplay();
    //ui->lineEdit_hex->setMaxLength(4);
    /*if (ui->checkBox_signed->checkState()) ui->lineEdit_dec->setMaxLength(6);
    else ui->lineEdit_dec->setMaxLength(5);*/

    svalue=ui->lineEdit_hex->text();

    value = (int)ui->lineEdit_hex->text().toUInt(0,16);
    SetValue(value, REG_CURRENT);
}

/****************************************************************************

    Qt Signal Handler

    control:    radioButton_32bit
    signal:     pressed

    The radio button for 32 bit mode has been selected (aka pressed)

****************************************************************************/
void HexTool::on_radioButton_32bit_clicked()
{
    int bitsize;
    QString svalue;
    int value;

    bitsize=GetNumBits();
    if (bitsize!=32 && ui->checkBox_signed->checkState() && optsignext)
    {

        switch (bitsize)
        {
            case 8: svalue="FFFFFF"+ui->lineEdit_hex->text();     break;
            case 16: svalue="FFFF"+ui->lineEdit_hex->text();    break;
        }
        SetNumBits(NUMBITS_32);
        value=(int)svalue.toUInt(0,16);
        SetValue(value, REG_CURRENT);
#ifdef QT_DEBUG
        qDebug() << "changed value:" << QString::number(value);
#endif
    }
    else
    {
        SetNumBits(NUMBITS_32);
    }
    UpdateDisplay();
    //ui->lineEdit_hex->setMaxLength(8);
    if (ui->checkBox_signed->checkState()) ui->lineEdit_dec->setMaxLength(11);
    else ui->lineEdit_dec->setMaxLength(10);
}
/****************************************************************************

    Qt Signal Handler

    control:    pushButton_not
    signal:     pressed

    The NOT button has been pressed

****************************************************************************/
void HexTool::on_pushButton_not_pressed()
{
    RegisterOperation(REG_CURRENT, REG_CURRENT, REG_CURRENT, OP_INVERSE);
    UpdateDisplay();
}

/****************************************************************************

   Qt Signal Handler

    control:    checkBox_signed
    signal:     toggled

    The checkbox for signed has been toggled for signed/unsigned operation

****************************************************************************/
void HexTool::on_checkBox_signed_toggled(bool isChecked)
{
    eSigned sig;

    if (isChecked)
        sig=SIGNED;
    else
        sig=UNSIGNED;

    SetSigned(sig);
    /*if (isChecked)
    {
        switch (GetNumBits())
        {
            case NUMBITS_8:  ui->lineEdit_dec->setMaxLength(4); break;
            case NUMBITS_16: ui->lineEdit_dec->setMaxLength(6); break;
            case NUMBITS_32: ui->lineEdit_dec->setMaxLength(11); break;
        }
    }*/
    UpdateDisplay();
    /*if (!isChecked)
    {
        switch (GetNumBits())
        {
            case NUMBITS_8:  ui->lineEdit_dec->setMaxLength(3); break;
            case NUMBITS_16: ui->lineEdit_dec->setMaxLength(5); break;
            case NUMBITS_32: ui->lineEdit_dec->setMaxLength(10); break;
        }
    }*/
    QString svalue;
    svalue=ui->lineEdit_hex->text();
    int value;
    value = (int)ui->lineEdit_hex->text().toUInt(0,16);
    SetValue(value, REG_CURRENT);
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton_shiftRight
    signal:     pressed

    The right shift button has been pressed to shift the number 1 place

****************************************************************************/
void HexTool::on_pushButton_shiftLeft_pressed()
{
    RegisterOperation(REG_CURRENT, REG_CURRENT, REG_CURRENT, OP_LEFT_SHIFT);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton_shiftRight
    signal:     pressed

    The right shift button has been pressed to shift the number 1 place

****************************************************************************/
void HexTool::on_pushButton_shiftRight_pressed()
{
    RegisterOperation(REG_CURRENT, REG_CURRENT, REG_CURRENT, OP_RIGHT_SHIFT);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton_bitNumbering
    signal:     pressed

    Toggle the labelling of the bits from zero based to 1 based

****************************************************************************/
void HexTool::on_pushButton_bitNumbering_pressed()
{
    static bool zeroBased = false;

    if (zeroBased)
    {
        zeroBased = false;
        ui->pushButton_bitNumbering->setText("1");
        ui->label_bit4->setText("4");
        ui->label_bit8->setText("8");
        ui->label_bit12->setText("12");
        ui->label_bit16->setText("16");
        ui->label_bit20->setText("20");
        ui->label_bit24->setText("24");
        ui->label_bit28->setText("28");
        ui->label_bit32->setText("32");
    }
    else
    {
        zeroBased = true;
        ui->pushButton_bitNumbering->setText("0");
        ui->label_bit4->setText("3");
        ui->label_bit8->setText("7");
        ui->label_bit12->setText("11");
        ui->label_bit16->setText("15");
        ui->label_bit20->setText("21");
        ui->label_bit24->setText("23");
        ui->label_bit28->setText("27");
        ui->label_bit32->setText("31");
    }
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton_endianSwap
    signal:     pressed

    Toggles the endian between Big and Little endian

****************************************************************************/
void HexTool::on_pushButton_endianSwap_pressed()
{
    unsigned int value = GetValue(REG_CURRENT);
    unsigned int newValue = 0;

    switch (GetNumBits())
    {
      case 32:
        {
          newValue |= (value & 0x000000ff) << 24;
          newValue |= (value & 0x0000ff00) <<  8;
          newValue |= (value & 0x00ff0000) >>  8;
          newValue |= (value & 0xff000000) >> 24;
        }
        break;

      case 16:
        {
          newValue |= (value & 0x000000ff) << 8;
          newValue |= (value & 0x0000ff00) >> 8;
        }
        break;

      default:
      case 8:
        // don't do anything!
        newValue = value;
        break;
    }
    SetValue(newValue, REG_CURRENT);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton increase and decrease
    signal:     clicked

    increases decreases value by one

****************************************************************************/
void HexTool::on_pushButton_increase_clicked()
{
    int value = GetValue(REG_CURRENT);
    value++;
    SetValue(value, REG_CURRENT);
    UpdateDisplay();
}

void HexTool::on_pushButton_decrease_clicked()
{
    int value = GetValue(REG_CURRENT);
    value--;
    SetValue(value, REG_CURRENT);
    UpdateDisplay();
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton_neg
    signal:     clicked

    Makes the decimal value negativ or positive in signed mode

****************************************************************************/
void HexTool::on_pushButton_neg_clicked()
{
    if (ui->checkBox_signed->isChecked()){

        QString decimal=ui->lineEdit_dec->text();

        if (decimal.left(1)!="-")
        ui->lineEdit_dec->setText("-" + decimal);
        else ui->lineEdit_dec->setText(decimal.mid(1,decimal.count()-1));
        //UpdateDisplay();
    } 
}

/****************************************************************************

    Qt Signal Handler

    control:    radioButton_register[A|B|C]
    signal:     clicked

    Select the current register that is being displayed

****************************************************************************/
void HexTool::on_radioButton_registerA_clicked()
{
    SetRegister(REG_A);
    UpdateDisplay();
}

void HexTool::on_radioButton_registerB_clicked()
{
    SetRegister(REG_B);
    UpdateDisplay();
}

void HexTool::on_radioButton_registerC_clicked()
{
    SetRegister(REG_C);
    UpdateDisplay();
}

/****************************************************************************

    This is the generic function to handle register operations after the user
    has clicked on one of the buttons.

****************************************************************************/
int HexTool::HandleOperation
(
    eOpType op
)
{
    // Perform the operation on the registers
    int retVal = RegisterOperation(REG_A, REG_B, REG_C, op);

    // change the display to show register C
    SetRegister(REG_C);
    ui->radioButton_registerC->setChecked(true);

    // Update the display
    UpdateDisplay();

    return (retVal);
}

/****************************************************************************

    Qt Signal Handler

    control:    pushButton_XXX
    signal:     pressed

    Where XXX is a register operation like AND, OR, XOR, ...
    These handlers just pass the processing to the generic HandleOperation
    function.

****************************************************************************/
void HexTool::on_pushButton_AND_pressed()
{
    HandleOperation(OP_AND);
}

void HexTool::on_pushButton_OR_pressed()
{
    HandleOperation(OP_OR);
}

void HexTool::on_pushButton_XOR_pressed()
{
    HandleOperation(OP_XOR);
}

void HexTool::on_pushButton_plus_pressed()
{
    HandleOperation(OP_ADD);
}

void HexTool::on_pushButton_minus_pressed()
{
    HandleOperation(OP_SUBTRACT);
}

void HexTool::on_pushButton_multiply_pressed()
{
    HandleOperation(OP_MULTIPLY);
}

void HexTool::on_pushButton_divide_pressed()
{
    HandleOperation(OP_DIVIDE);
}




/****************************************************************************

    This generic handler deals with the user editing the IP Address edit
    boxes.
    It will automatically move the focus to the next control if the user
    enters a 3 digit number, because each part of an IP address is always
    in the range 0...255.
    Users are also forbidden from entering numbers greater than 255.

****************************************************************************/
void HexTool::HandleIpAddrEditBox
(
    int field,
    unsigned int val
)
{
    if (field < 4)
    {
        if ((val & 0xFFFFFF00) == 0)
        {
            int currentValue    = GetValue(REG_CURRENT);    // the current value stored in the current register
            int newValue        = 0;                        // the new value that we want to to set the register to

            switch (field)   // which byte was modified (i.e. which ASCI edit box)
            {
                case 0: newValue = (currentValue & 0xFFFFFF00) | (val      ); break;
                case 1: newValue = (currentValue & 0xFFFF00FF) | (val <<  8); break;
                case 2: newValue = (currentValue & 0xFF00FFFF) | (val << 16); break;
                case 3: newValue = (currentValue & 0x00FFFFFF) | (val << 24); break;
            }

            SetValue(newValue, REG_CURRENT);
            UpdateDisplay();

            // If the value has changed, skip to the next character
            if ((newValue != currentValue) && (val > 99))
            {
                QLineEdit * nextCtrl;
                switch (field)
                {
                case 0:
                    switch (GetNumBits())
                    {
                    case NUMBITS_8:  nextCtrl = ui->lineEdit_ipaddress0; break;
                    case NUMBITS_16: nextCtrl = ui->lineEdit_ipaddress1; break;
                    case NUMBITS_32: nextCtrl = ui->lineEdit_ipaddress3; break;
                    }
                    break;
                case 1: nextCtrl = ui->lineEdit_ipaddress0; break;
                case 2: nextCtrl = ui->lineEdit_ipaddress1; break;
                case 3: nextCtrl = ui->lineEdit_ipaddress2; break;
                }
                nextCtrl->setFocus();
                nextCtrl->setSelection(0,10);
            }
        }
        else
        {
            // refuse to accept more than 255 in an IP Addr edit box
            UpdateDisplay();
        }
    }
}


/****************************************************************************

    Qt Signal Handler

    control:    lineEdit_ipaddressX
    signal:     textChanged

    When the users edits the ip address fields, these handlers pass control
    to the generic HandleIpAddrEditBox function.

****************************************************************************/
void HexTool::on_lineEdit_ipaddress0_textChanged()
{
    HandleIpAddrEditBox(0, ui->lineEdit_ipaddress0->text().toUInt());
}

void HexTool::on_lineEdit_ipaddress1_textChanged()
{
    HandleIpAddrEditBox(1, ui->lineEdit_ipaddress1->text().toUInt());
}

void HexTool::on_lineEdit_ipaddress2_textChanged()
{
    HandleIpAddrEditBox(2, ui->lineEdit_ipaddress2->text().toUInt());
}

void HexTool::on_lineEdit_ipaddress3_textChanged()
{
    HandleIpAddrEditBox(3, ui->lineEdit_ipaddress3->text().toUInt());
}


/****************************************************************************

    The generic handler called when the users types into the ASCII edit boxes.
    The UI is programmed to only allow 1 character using the form designer.
    This function automatically moves focus to the next ASCII edit control,
    wrapping back to the first.

****************************************************************************/
void HexTool::HandleAsciiEditBox
(
    int field,
    unsigned int val
)
{
    if (field < 4)
    {
        int newByte         = val & 0x000000FF;// the byte that was just typed in the ASCII edit box
        int currentValue    = GetValue(REG_CURRENT);    // the current value stored in the current register
        int newValue        = 0;                        // the new value that we want to to set the register to

        switch (field)   // which byte was modified (i.e. which ASCI edit box)
        {
            case 0: newValue = (currentValue & 0xFFFFFF00) | (newByte      ); break;
            case 1: newValue = (currentValue & 0xFFFF00FF) | (newByte <<  8); break;
            case 2: newValue = (currentValue & 0xFF00FFFF) | (newByte << 16); break;
            case 3: newValue = (currentValue & 0x00FFFFFF) | (newByte << 24); break;
        }

        SetValue(newValue, REG_CURRENT);
        UpdateDisplay();

        // If the value has changed, skip to the next character
        if (newValue != currentValue)
        {
            QLineEdit * nextCtrl;
            switch (field)
            {
            case 0:
                switch (GetNumBits())
                {
                case NUMBITS_8:  nextCtrl = ui->lineEdit_ascii0; break;
                case NUMBITS_16: nextCtrl = ui->lineEdit_ascii1; break;
                case NUMBITS_32: nextCtrl = ui->lineEdit_ascii3; break;
                }
                break;
            case 1: nextCtrl = ui->lineEdit_ascii0; break;
            case 2: nextCtrl = ui->lineEdit_ascii1; break;
            case 3: nextCtrl = ui->lineEdit_ascii2; break;
            }
            nextCtrl->setFocus();
            nextCtrl->setSelection(0,1);
        }
    }
}
/****************************************************************************

    Qt Signal Handler

    control:    lineEdit_asciiX
    signal:     textChanged

    Passes control on to a generic handler after extracting the new ASCII
    value that was entered.

****************************************************************************/
void HexTool::on_lineEdit_ascii0_textChanged()
{
    QString text;
    text = ui->lineEdit_ascii0->text();
    int val = (int)text[0].unicode();
    HandleAsciiEditBox(0, val);
}
void HexTool::on_lineEdit_ascii1_textChanged()
{
    QString text;
    text = ui->lineEdit_ascii1->text();
    int val = (int)text[0].unicode();
    HandleAsciiEditBox(1, val);
}
void HexTool::on_lineEdit_ascii2_textChanged()
{
    QString text;
    text = ui->lineEdit_ascii2->text();
    int val = (int)text[0].unicode();
    HandleAsciiEditBox(2, val);
}
void HexTool::on_lineEdit_ascii3_textChanged()
{
    QString text;
    text = ui->lineEdit_ascii3->text();
    int val = (int)text[0].unicode();
    HandleAsciiEditBox(3, val);
}
/****************************************************************************

    clearAll function clears values from all register

****************************************************************************/
void HexTool::clearAll()
{
    int value;
    value=0;
    SetValue(value, REG_A);
    SetValue(value, REG_B);
    SetValue(value, REG_C);
    UpdateDisplay();
}
/****************************************************************************

    About MessageBox

****************************************************************************/
void HexTool::about()
{
    QPixmap logo(":/new/ownresource/hextool48.png");
    //QMessageBox::about(this,tr("HexTool Maemo Version 1.7.1maemo1"),tr("Ported and code changes by Halftux.\nBased on Version 1.7.0.0 by Ryan Harkin."));
    QMessageBox about;

        about.setWindowTitle("HexTool Maemo Version 1.7.1maemo2");
        about.setText("Ported and code changes by Halftux.\nBased on Version 1.7.0.0 by Ryan Harkin.");
        //about.setInformativeText();
        about.setStandardButtons(QMessageBox::Ok);
        about.setIconPixmap(logo);
        about.setDefaultButton(QMessageBox::Ok);
        about.show();
        about.exec();

    //QMessageBox::about.setIconPixmap(QPixMap(":/new/onwresource/hextool48.png"));
}
/****************************************************************************

    Help MessageBox

****************************************************************************/
void HexTool::help()
{
    QMessageBox::information(this,tr("Help"),
                             "You can put different numbers in register <b>A</b> and <b>B</b> and make logical and mathematical operations. The result is shown in register <b>C</b>.<br>The <i>arrow</i> buttons: shift operations (current register).<br>The <i>flip</i> button: endian swap (hexadecimal value).<br>The <i>not</i> button: invert (binary).<br>The small <i>+</i> and <i>-</i> button: increases/decreases (dezimal value).<br>The <i>one/zero</i> button: change the label 0/1 (binary).<br>The <i>neg</i> button: inverse (dezimal value [signed])");
}
/****************************************************************************

    Save function

    Saves status of radiobuttons, checkbox, all register values and
    the options in an ini file

****************************************************************************/
void HexTool::save_ini()
{
    QSettings settings(QDir::homePath() + "/.hextool/settings.ini", QSettings::IniFormat);
    settings.beginGroup("General");
    settings.setValue("signed", ui->checkBox_signed->isChecked());
    settings.setValue("bitsize", ui->rbuttonGroup_bitsize->checkedId());
    settings.setValue("lastreg", ui->rbuttonGroup_reg->checkedId());
    settings.setValue("valuerega", GetValue(REG_A));
    settings.setValue("valueregb", GetValue(REG_B));
    settings.setValue("valueregc", GetValue(REG_C));
    settings.endGroup();
    settings.beginGroup("Options");
    settings.setValue("optsl", optsl);
    settings.setValue("optsignext", optsignext);
    settings.endGroup();


#if defined(Q_WS_MAEMO_5)
    QMaemo5InformationBox::information(this, "settings saved", QMaemo5InformationBox::DefaultTimeout);
#endif
}
/****************************************************************************

    Load function

    Loads status of radiobuttons, checkbox, all register values and
    the options from an ini file

****************************************************************************/
void HexTool::load_ini()
{
    QFileInfo fi(QDir::homePath() + "/.hextool/settings.ini");
    if (!fi.exists())
    {
        #if defined(Q_WS_MAEMO_5)
            QMaemo5InformationBox::information(this, "no file found", QMaemo5InformationBox::DefaultTimeout);
        #else
            QMessageBox::warning(this, "File not found","The settings file was not found.",QMessageBox::Ok,0);
        #endif

    }
    else
    {
        int bitsize;
        int reg_id;
        int valueA;
        int valueB;
        int valueC;
        QSettings settings(QDir::homePath() + "/.hextool/settings.ini", QSettings::IniFormat);
        settings.beginGroup("General");
        ui->checkBox_signed->setChecked(settings.value("signed").toBool());
        bitsize=settings.value("bitsize").toInt();
        reg_id=settings.value("lastreg").toInt();
        valueA=settings.value("valuerega").toInt();
        valueB=settings.value("valueregb").toInt();
        valueC=settings.value("valueregc").toInt();
        settings.endGroup();
        settings.beginGroup("Options");
        optsl=settings.value("optsl").toBool();
        optsignext=settings.value("optsignext").toBool();
        settings.endGroup();

        switch (bitsize)
        {
        case 32:
           ui->radioButton_32bit->setChecked(true);
           on_radioButton_32bit_clicked();
           break;
        case 16:
           ui->radioButton_16bit->setChecked(true);
           on_radioButton_16bit_clicked();
           break;
        case 8:
           ui->radioButton_8bit->setChecked(true);
           on_radioButton_8bit_clicked();
           break;
        default:
           qDebug() << "switch:" << QString::number(bitsize);
        }
        switch (reg_id)
        {
        case 1:
           ui->radioButton_registerA->setChecked(true);
           SetRegister(REG_A);
           break;
        case 2:
           ui->radioButton_registerB->setChecked(true);
           SetRegister(REG_C);
           break;
        case 3:
           ui->radioButton_registerC->setChecked(true);
           SetRegister(REG_B);
           break;
        default:
           qDebug() << "switch:" << QString::number(reg_id);
        }

        SetValue(valueA, REG_A);
        SetValue(valueB, REG_B);
        SetValue(valueC, REG_C);
        UpdateDisplay();

#if defined(Q_WS_MAEMO_5)
        QMaemo5InformationBox::information(this, "settings loaded", QMaemo5InformationBox::DefaultTimeout);
#endif
    }
}
/****************************************************************************

    QDialog: Options

    Shows the option checkboxes and returns bool statements

****************************************************************************/
void HexTool::optDialog()
{
    optionsDialog showopt(optsl,optsignext,this);
    //showopt.exec();
    if (showopt.exec()==optionsDialog::Accepted)
    {
        optsl=showopt.saveload();
        optsignext=showopt.signext();

    }
}
